////////////////////////////////////////////////////////////////////////////
// #02012634
//
// Copyright (C) 2002 Albert Vik
//
// E-mail: AlbertVik@operamail.com
//
// This source code is part of FLAC project. This source code is distributed 
// under the same policy as the FLAC project constitute except one addition:
// any modifications of this file and the source code in this file for 
// purposes other than for private use should be sanctioned by the author.
//
// The latest release of this source code is available on the  
// FLAC project home page: http://flac.sourceforge.net or
// directly from the author: AlbertVik@operamail.com
// 
////////////////////////////////////////////////////////////////////////////

#include "FLAC_Apollo.h"

////////////////////////////////////////////////////////////////////////////
//
// Class CFLACInfo - FLAC File Detailed Information
//
////////////////////////////////////////////////////////////////////////////

CFLACInfo::CFLACInfo()
{
    m_StrFileName = NULL;

    Clear();

}  // CFLACInfo::CFLACInfo()



CFLACInfo::CFLACInfo(char * filename)
{
    m_StrFileName = NULL;

    Clear();

    GetInfo(filename);

}  // CFLACInfo::CFLACInfo()



CFLACInfo::~CFLACInfo()
{
   Clear();

}  // CFLACInfo::~CFLACInfo()



void CFLACInfo::Clear()
{
    if(m_StrFileName)  delete [] m_StrFileName;
 
    *m_StrFileDate=0; *m_StrPlayingTime=0; *m_StrChannels=0; *m_StrMD5=0;
    m_FileSize=0; m_PlayingTime=0; m_SampleWidth=0; m_Compression=0;
    m_BitRatePCM=0; m_BitRateFLAC=0;

    m_IsValid = FALSE;

}  // CFLACInfo::Clear()



BOOL CFLACInfo::GetInfo(char * filename)
{
    Clear();

    if (strstr(filename,"://") != NULL) return FALSE;   // No URLs

    if( !(m_StrFileName = new char[strlen(filename)+1]) )
    {
        return m_IsValid = FALSE;
    }

    m_StrFileName = strcpy( m_StrFileName, filename );
	
    // Get size of file
    HANDLE hFile = CreateFile(filename,GENERIC_READ,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);

    if(hFile == INVALID_HANDLE_VALUE) return FALSE;    // Error opening file
    m_FileSize = GetFileSize(hFile,NULL);

    FILETIME  file_time; GetFileTime(hFile,&file_time, NULL, NULL);
    SYSTEMTIME sys_time; FileTimeToSystemTime( &file_time, &sys_time );
    GetDateFormat( NULL, DATE_LONGDATE,  &sys_time, NULL, m_StrFileDate, sizeof(m_StrFileDate) );
    GetTimeFormat( NULL, TIME_NOSECONDS, &sys_time, NULL, strcat(m_StrFileDate," ")+strlen(m_StrFileDate), sizeof(m_StrFileDate) );

    CloseHandle(hFile);

    FLAC__StreamMetaData_StreamInfo si;

    if( !FLAC__utility_get_streaminfo(filename, &si) )
    {
        return m_IsValid = FALSE;
    }

    m_SampleRate=si.sample_rate;
    m_Channels=si.channels;
    m_BitsPerSample=si.bits_per_sample;

    m_SampleWidth = si.bits_per_sample ==  8 ? 1 : 
                   (si.bits_per_sample == 16 ? 2 : 3);
	
    m_TotalSamples = (unsigned)si.total_samples;

    if( m_TotalSamples == 0 )   // It can be 0, meaning 'unknown'
    {
        // It is possible to estimate it assuming 50% compression
        m_TotalSamples = FileSize() * 2 / Channels() / SampleWidth();

        m_StrApproxMark = "n/a, nearly ";
    } else
        m_StrApproxMark = "";

    if( !TotalSamples() ) 
    {
        return m_IsValid = FALSE;
    } 
		
    m_Compression = (double)TotalSamples() *
                    (double)Channels() * 
                    (double)SampleWidth() / 
                    (double)FileSize();

    // Milliseconds
    m_PlayingTime = (int)(
                    (double)1000 * 
                    (double)TotalSamples() / 
                    (double)SampleRate() );

    if( !m_PlayingTime ) m_PlayingTime++;  // PlayingTime>0 if TotalSamples()>0

    sprintf( StrPlayingTime(), "%lu:%.2lf", PlayingTime()/60000, 
                                           (PlayingTime()%60000)/1000.0 );
	
    if     ( Channels() == 1 ) sprintf( m_StrChannels, "mono"   );
    else if( Channels() == 2 ) sprintf( m_StrChannels, "stereo" );
    else                       sprintf( m_StrChannels, "%ul", si.channels ); 

    // Uncompressed - as in PCM
    m_BitRatePCM = SampleRate() * Channels() * BitsPerSample();

    // Real FLAC - for correct compression ratio calculation
    m_BitRateFLAC = (int)(
                    (double)SampleRate() * 
                    (double)FileSize() * 
                    (double)8.0 / 
                    (double)TotalSamples() );

    m_MinBlocksize=si.min_blocksize; m_MaxBlocksize=si.max_blocksize;
    m_MinFramesize=si.min_framesize; m_MaxFramesize=si.max_framesize;
			
    for( int i = 0; i < 16; i++ )  // FLAC__byte md5sum[16];
    {
        m_MD5Sum[i] = si.md5sum[i];
        sprintf( m_StrMD5 + i*2, "%02X ", (int)si.md5sum[i] );
    }	

   return m_IsValid = TRUE;

}  // CFLACInfo::GetInfo()


BOOL CFLACInfo::IsMD5Signed()
{
    unsigned sum = 0;

    for( int i = 0; i < 16; i++ )  // FLAC__byte md5sum[16];
    {
        sum += (unsigned)m_MD5Sum[i];
    }	

    return sum ? TRUE : FALSE;

}  // CFLACInfo::IsMD5Signed()

