////////////////////////////////////////////////////////////////////////////
// #02012635
//
// Copyright (C) 2002 Albert Vik
//
// E-mail: AlbertVik@operamail.com
//
// This source code is part of FLAC project. This source code is distributed 
// under the same policy as the FLAC project constitute except one addition:
// any modifications of this file and the source code in this file for 
// purposes other than for private use should be sanctioned by the author.
//
// The latest release of this source code is available on the  
// FLAC project home page: http://flac.sourceforge.net or
// directly from the author: AlbertVik@operamail.com
// 
////////////////////////////////////////////////////////////////////////////

#include "FLAC_Apollo.h"

////////////////////////////////////////////////////////////////////////////
//
// Class CFLACInfoGUI - FLAC Info GUI Dialogs
//
////////////////////////////////////////////////////////////////////////////

int CFLACInfoGUI::AdditionalInfoDialog( HINSTANCE hInstance, UINT nIDDialog )
{
    return DialogBoxParam( hInstance, MAKEINTRESOURCE( nIDDialog ),
               GetActiveWindow(), ::AdditionalInfoDialogProc, (LPARAM)this);

}  // CFLACInfoGUI::AdditionalInfoDialog()



BOOL CFLACInfoGUI::TestAudioData( HWND hwndDlg, BOOL *abort )
{
    if( !IsValid() )     return FALSE;
    if( !IsMD5Signed() ) return FALSE;

    CFLACDecoder decoder( StrFileName(), 0);

    if( !decoder.IsValid() ) return FALSE;
    
    
    HWND hwndPB = GetDlgItem( hwndDlg, IDC_MD5_PROGRESS );
					
    const int MAX_RANGE = 1000;
    SendMessage( hwndPB, PBM_SETRANGE32, 0, MAX_RANGE ); 

    unsigned bytes_in_chunk, bytes_processed = 0;

    do {  // NOTE: It's a right place to collect audio statistics.

       PumpMessages();  // To not lock Apollo while processing
       
       char *chunk;  

       // Process whole audio data to calculate MD5 signature of unencoded audio data
       bytes_in_chunk = decoder.GetNextAudioChunk( (void**)&chunk );

       bytes_processed += bytes_in_chunk;
	
       WPARAM progress = (WPARAM)
		   ( 
		   (double)MAX_RANGE * 
		   (double)bytes_processed / 
		   (double)TotalSamples() / 
		   (double)Channels() / 
		   (double)SampleWidth() 
		   );

       SendMessage(hwndPB, PBM_SETPOS, progress, 0); 

       } while( bytes_in_chunk && !*abort );

    // It report when the decoded MD5 signature does not match the one stored 
    // in the STREAMINFO block. MD5 checking is automatically turned off if 
    // there is no signature in the STREAMINFO block or when a seek is attempted. 

    BOOL state = FLAC__file_decoder_finish( decoder.GetDecoder() );

    if(!*abort) SendMessage(hwndDlg, WM_COMMAND, (state ? AUDIODATA_OK : AUDIODATA_ERROR), 0);
	
    return state;

}  // CFLACInfoGUI::TestAudioData()



BOOL CFLACInfoGUI::AdditionalInfoDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{

  HINSTANCE DLLInstance = (HINSTANCE)GetWindowLong( hwndDlg, GWL_HINSTANCE );
	
    switch (uMsg)
    {
        case WM_INITDIALOG:
            {
            DlgPrintF( hwndDlg, IDC_MD5_STRING, StrMD5() );
            DlgPrintF( hwndDlg, IDC_FILENAME, "%s", StrFileName() );
            DlgPrintF( hwndDlg, IDC_TOTAL_SAMPLES, "%s%lu", StrApproxMark(), TotalSamples() );
            DlgPrintF( hwndDlg, IDC_BLOCKSIZE, "min %lu %s, max %lu %s", MinBlocksize(), "samples", MaxBlocksize(), "samples" ); 
            DlgPrintF( hwndDlg, IDC_FRAMESIZE, "min %lu %s, max %lu %s", MinFramesize(), "bytes",   MaxFramesize(), "bytes" ); 
            DlgPrintF( hwndDlg, IDC_CHANNELS, "%s", StrChannels() );
            DlgPrintF( hwndDlg, IDC_BPS, "%lu", BitsPerSample() );
            DlgPrintF( hwndDlg, IDC_SAMPLE_RATE, "%lu Hz", SampleRate() );
            DlgPrintF( hwndDlg, IDC_BITRATE, "%lu kb/s  (%lu kb/s)", BitRatePCM()/1000, BitRateFLAC()/1000 );
            DlgPrintF( hwndDlg, IDC_PLAYINGTIME, "%s%s", StrApproxMark(), StrPlayingTime() );
            DlgPrintF( hwndDlg, IDC_COMPRESSION, "1 : %.3lf,  %.0lf%%", Compression(),  100/Compression() );
            DlgPrintF( hwndDlg, IDC_FILE_SIZE, "%lu bytes", FileSize() );
            DlgPrintF( hwndDlg, IDC_FILE_DATE, StrFileDate() );            
            }
            return (TRUE);

        case WM_DESTROY:

            return (FALSE);

        case WM_HELP:

            SendMessage(hwndDlg, WM_COMMAND, IDABOUT, 0);
            return (FALSE); 

        case WM_COMMAND:

            UINT uCmdId = (UINT) wParam;

            switch (uCmdId)
            {
                case IDTESTAUDIODATA:
                    
                    if( IsMD5Signed() ) 
                    {
                        InitCommonControls();
                        int status = DialogBoxParam(DLLInstance, MAKEINTRESOURCE(IDD_TESTAUDIODATA_PROGRESS), hwndDlg, ::TestAudiodataDialogProc, (LPARAM)this);
                        if( status != IDCANCEL )
                        {
                            UINT choise;
                            if( status == AUDIODATA_OK ) choise = DlgMessageBox( DLLInstance, IDD_MD5_OK,    IDS_MD5_OK );
                            else                         choise = DlgMessageBox( DLLInstance, IDD_MD5_ERROR, IDS_MD5_ERROR );

                            if( choise == IDUSER ) DlgMessageBox( DLLInstance, IDD_DESCRIPTION, IDS_DESCRIPTION );
                        }
                    } else
                        DlgMessageBox( DLLInstance, IDD_MD5_NOTSIGNED, IDS_MD5_NOTSIGNED );
                                        
                    break;

                case IDABOUT:
                    {
                    UINT choise = DlgMessageBox( DLLInstance, IDD_ABOUT, IDS_ABOUT );
                    if( choise == IDUSER ) DlgMessageBox( DLLInstance, IDD_DESCRIPTION, IDS_DESCRIPTION );
                    }
                    break;

                case IDOK:
                    EndDialog(hwndDlg, IDOK);
                    break;
                case IDCANCEL:
                    EndDialog(hwndDlg, IDCANCEL);
                    break;
            }
            return (TRUE);
    }

    return (FALSE);

} // CFLACInfoGUI::AdditionalInfoDialogProc()



BOOL CFLACInfoGUI::TestAudiodataDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{

    static BOOL abort;

    switch (uMsg)
    {
        case WM_INITDIALOG:
            
            ShowWindow(hwndDlg, SW_SHOW);
            abort = FALSE;
            TestAudioData(hwndDlg, &abort);
            return (TRUE);

        case WM_DESTROY:

            return (FALSE);

        case WM_COMMAND:

            UINT uCmdId = (UINT) wParam;

            switch (uCmdId)
            {
                case AUDIODATA_OK:
                    EndDialog(hwndDlg, AUDIODATA_OK);
                    break;
                case AUDIODATA_ERROR:
                    EndDialog(hwndDlg, AUDIODATA_ERROR);
                    break;
                case IDCANCEL:
                    abort = TRUE;
                    EndDialog(hwndDlg, IDCANCEL);
                    break;
            }
            return (TRUE);
    }

    return (FALSE);

}  // CFLACInfoGUI::TestAudiodataDialogProc()



////////////////////////////////////////////////////////////////////////////
//
// Win32 GUI Callback Stubs
//
////////////////////////////////////////////////////////////////////////////

BOOL CALLBACK AdditionalInfoDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{

    if( uMsg == WM_INITDIALOG ) SetWindowLong( hwndDlg, DWL_USER, (LONG)lParam );

    CFLACInfoGUI *info = (CFLACInfoGUI*)GetWindowLong( hwndDlg, DWL_USER);
  
    return info ? info->AdditionalInfoDialogProc( hwndDlg, uMsg, wParam, lParam ) : FALSE;

}  // ::AdditionalInfoDialogProc()



BOOL CALLBACK TestAudiodataDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{

    if( uMsg == WM_INITDIALOG ) SetWindowLong( hwndDlg, DWL_USER, (LONG)lParam );

    CFLACInfoGUI *info = (CFLACInfoGUI*)GetWindowLong( hwndDlg, DWL_USER);
  
    return info ? info->TestAudiodataDialogProc( hwndDlg, uMsg, wParam, lParam ) : FALSE;

}  // ::TestAudiodataDialogProc()


