////////////////////////////////////////////////////////////////////////////
// #02012632
//
// Copyright (C) 2002 Albert Vik
//
// E-mail: AlbertVik@operamail.com
//
// This source code is part of FLAC project. This source code is distributed 
// under the same policy as the FLAC project constitute except one addition:
// any modifications of this file and the source code in this file for 
// purposes other than for private use should be sanctioned by the author.
//
// The latest release of this source code is available on the  
// FLAC project home page: http://flac.sourceforge.net or
// directly from the author: AlbertVik@operamail.com
// 
////////////////////////////////////////////////////////////////////////////

#include "FLAC_Apollo.h"

////////////////////////////////////////////////////////////////////////////
//
// Class CFLACPlugin
//
////////////////////////////////////////////////////////////////////////////

char *CFLACPlugin::GetDescription()
{
    return "FLAC Lossless Audio Codec Decoder";

}  // CFLACPlugin::GetDescription()



CInputDecoder *CFLACPlugin::Open(char *filename,int audioDataOffset)
{
    // returning NULL means trouble initializing decoder
    return new CFLACDecoder(filename,audioDataOffset);

}  // CFLACPlugin::Open()



BOOL CFLACPlugin::IsSuitableFile(char *filename,int audioDataOffset)
{
    if (strstr(filename,"://") != NULL) return FALSE;   // No URLs

    char *extensionBegin = strrchr(filename,'.');

    if (extensionBegin == NULL) return FALSE;

    if (!stricmp(extensionBegin,".flac")) return TRUE;
    if (!stricmp(extensionBegin,".fla" )) return TRUE;

    return FALSE;

}  // CFLACPlugin::IsSuitableFile()



char *CFLACPlugin::GetExtensions()
{
    // Returns a string containing all supported file extensions + their descriptions.
    // Format: "description1\0*.ext1;*.ext2;*.ext3\0description2\0*.ext1;*.ext2;*.ext3\0\0"
    // This information will be used in open/add dialogs and with shell integration

    return "FLAC Lossless Audio Codec\0*.FLAC;*.FLA\0\0";

}  // CFLACPlugin::GetExtensions()



void CFLACPlugin::About()
{
    UINT choise = DlgMessageBox( m_hDLLInstance, IDD_ABOUT, IDS_ABOUT );

    if( choise == IDUSER ) DlgMessageBox( m_hDLLInstance, IDD_DESCRIPTION, IDS_DESCRIPTION );

}  // CFLACPlugin::About()



void CFLACPlugin::Config()
{
    UINT choise = DlgMessageBox( m_hDLLInstance, IDD_CONFIGURE, IDS_CONFIGURE );

    if( choise == IDUSER  ) DlgMessageBox( m_hDLLInstance, IDD_DESCRIPTION, IDS_DESCRIPTION );
    if( choise == IDABOUT ) About();

}  // CFLACPlugin::Config()



void CFLACPlugin::AdditionalInfo( char *filename,int audioDataOffset )
{
    CFLACInfoGUI info(filename);

    if( info.IsValid() ) 
        info.AdditionalInfoDialog( m_hDLLInstance, IDD_ADDITIONALINFO );

}  // CFLACPlugin::AdditionalInfo()



BOOL CFLACPlugin::GetInfo( char *filename, int audioDataOffset, TrackInfo *trackInfo )
{
    CFLACInfo info(filename);

    if ( !info.IsValid() ) return FALSE;

    strcpy(trackInfo->fileTypeDescription,"FLAC Lossless Audio Codec");

    trackInfo->seekable      = TRUE;
    trackInfo->hasEqualizer  = FALSE;  // Passing equalization to Apollo
    trackInfo->fileSize      = info.FileSize();
    trackInfo->playingTime   = info.PlayingTime();
    trackInfo->bitRate       = info.BitRateFLAC();
    trackInfo->sampleRate    = info.SampleRate();
    trackInfo->numChannels   = info.Channels();
    trackInfo->bitResolution = info.BitsPerSample();

    // Formatted track description for Apollo.
    // If the file contains information about the title you should
    // use it as the suggested title instead of the filebody

    CId3v1 tag(filename);

    if( tag.IsValid() ) 
        sprintf( trackInfo->suggestedTitle, "%s - %s", tag.artist, tag.title );
    else 
    {
        // Init trackInfo->suggestedTitle by file name
        char *extensionBegin = strrchr(filename,'.');
        if (extensionBegin == NULL) extensionBegin = filename + strlen(filename);

        char *pathEnd = strrchr(filename,'\\');
        if (pathEnd == NULL)
        {
            memcpy(trackInfo->suggestedTitle,filename,extensionBegin - filename);
            trackInfo->suggestedTitle[extensionBegin - filename] = 0;
        }
        else
        {
            memcpy(trackInfo->suggestedTitle,pathEnd + 1,extensionBegin - pathEnd - 1);
            trackInfo->suggestedTitle[extensionBegin - pathEnd - 1] = 0;
        }
    }

    return TRUE;

}  // CFLACPlugin::GetInfo()


