////////////////////////////////////////////////////////////////////////////
// #02012636
//
// Copyright (C) 2002 Albert Vik
//
// E-mail: AlbertVik@operamail.com
//
// This source code is part of FLAC project. This source code is distributed 
// under the same policy as the FLAC project constitute except one addition:
// any modifications of this file and the source code in this file for 
// purposes other than for private use should be sanctioned by the author.
//
// The latest release of this source code is available on the  
// FLAC project home page: http://flac.sourceforge.net or
// directly from the author: AlbertVik@operamail.com
// 
////////////////////////////////////////////////////////////////////////////

#include "FLAC_Apollo.h"

////////////////////////////////////////////////////////////////////////////
//
// Global Win32 Helpers
//
////////////////////////////////////////////////////////////////////////////

int DlgMessageBox( HINSTANCE hInstance, UINT nIDDlgItem )
{
    return DlgMessageBox( hInstance, nIDDlgItem, NULL );

}  // ::DlgMessageBox()



int DlgMessageBox( HINSTANCE hInstance, UINT nIDDlgItem, UINT nIDString )
{
    HWND hWnd = GetActiveWindow();

    return DialogBoxParam( hInstance, MAKEINTRESOURCE(nIDDlgItem), hWnd, ::DlgMessageBoxDialogProc, nIDString );

}  // ::DlgMessageBox()



BOOL CALLBACK DlgMessageBoxDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{

    switch (uMsg)
    {
        case WM_INITDIALOG:

            DlgPrintS(hwndDlg, IDC_TEXT, lParam);

            return (TRUE);

        case WM_DESTROY:
            
            return (FALSE);

        case WM_COMMAND:

            UINT uCmdId = (UINT) wParam;

            switch (uCmdId)
            {
                case IDUSER:
                    EndDialog(hwndDlg, IDUSER);
                    break;

                case IDABOUT:
                    EndDialog(hwndDlg, IDABOUT);
                    break;

                case IDOK:
                    EndDialog(hwndDlg, IDOK);
                    break;

                case IDCANCEL:
                    EndDialog(hwndDlg, IDCANCEL);
                    break;
            }

            return (TRUE);
    }

    return (FALSE);

} // ::DlgMessageBoxDialogProc()



int LoadText( HINSTANCE hInstance, UINT uID, LPTSTR lpBuffer, int nBufferMax )
{
    const int MAX_RESOURCE_STRING_LENGTH = 4096;

    char *string = new char[MAX_RESOURCE_STRING_LENGTH];

    if( !string ) return 0;

    int source_count = LoadString( hInstance, uID, string, nBufferMax );
    int dest_count = 0;

    for( int i = 0; i < source_count && string[i] && dest_count < nBufferMax-1; i++ )

        if( string[i]=='\n' && (string[i-1]!='\r' || i==0) ) 
        {
            lpBuffer[dest_count++] = '\r';
            if( dest_count < nBufferMax-1 ) lpBuffer[dest_count++] = '\n';
        } 
        else 
            lpBuffer[dest_count++] = string[i];

    lpBuffer[dest_count] = 0;

    delete [] string;

    return dest_count;

} // ::LoadText()

 

BOOL DlgPrintS( HWND hwndDlg, UINT nIDDlgItem, UINT nIDString )
{
    const int MAX_RESOURCE_STRING_LENGTH = 4096;

    char *string = new char[MAX_RESOURCE_STRING_LENGTH];

    HINSTANCE APPInstance = (HINSTANCE)GetWindowLong( hwndDlg, GWL_HINSTANCE );

    int count = LoadText( APPInstance, nIDString, string, MAX_RESOURCE_STRING_LENGTH );

    if( !count ) *string = 0;

    BOOL rc = SetDlgItemText( hwndDlg, nIDDlgItem, string  );
 
    if( string ) delete [] string;

    return rc; 

}  // ::DlgPrintS()



BOOL DlgPrintF(  HWND hwndDlg, UINT nIDDlgItem, const char *format, ...  )
{
    const int MAX_RESOURCE_STRING_LENGTH = 4096;

    char *string = new char [MAX_RESOURCE_STRING_LENGTH];

    va_list rest;  va_start( rest, format );     
    _vsnprintf( string, MAX_RESOURCE_STRING_LENGTH, format, rest );

    BOOL rc = SetDlgItemText( hwndDlg, nIDDlgItem, string  );

    if( string ) delete [] string;    

    return rc;

}  // ::DlgPrintF()



BOOL PumpMessages( void )
{
    MSG msg;

    while ( PeekMessage( &msg, NULL, NULL, NULL, PM_NOREMOVE ) ) 
    {
        if (!GetMessage( &msg, NULL, NULL, NULL))  return FALSE;

        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return TRUE;

}  // ::PumpMessages()

